/* 
 * CSE142, 99 Winter
 * Homework 5
 * Valentine's Day
 *
 * Author:     Stephen Nickels
 * Student ID: 9723651
 * Section:    BE
 *	
 */

#include "gp142.h"
#include <math.h>
#include <time.h>
	/* <time.h> is here only so that one function can be called
	to seed the random number generator.
	Put it here in your program and forget about it, 
	just in case you decide later you need to use random numbers.
	*/

#define TRUE    		1
#define FALSE   		0

/*
 * The state, or status you are in determines what happens when you click 
 * the card:
 * ADD TEXT	- Click on the card to add text.
 * ADD SHAPE   -- you click the card, and a new object is placed there.
 *                You get into this mode by clicking an ADD button.
 *                After doing an ADD, you are put into SELECTED mode.
 * SELECTED    -- An object is selected. In this mode, the type 
 *                rotation applies to thecurrent object.
 * SELECT	- In this mode, the user is asked to click near an object
 *		  to select it.
 * BACKGROUND	- In this mode, the background color may be changed.
 */
#define STATUS_ADD_TEXT 1
#define STATUS_ADD_SHAPE 2
#define STATUS_SELECT 3
#define STATUS_SELECTED 4
#define STATUS_BACKGROUND 5

#define BUTTON_HEIGHT  	35
#define BUTTON_WIDTH 	100
#define COLOR_SIDE 23

#define NUM_BUTTONS 	5

/* Position of the card in the window */
#define PAGE_LLX (-GP142_XMAX + BUTTON_WIDTH +60)
#define PAGE_LLY (-GP142_YMAX + 100)
#define PAGE_WIDTH (GP142_XMAX - PAGE_LLX - 80)
#define PAGE_HEIGHT (GP142_YMAX - PAGE_LLY - 50)

/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0
#define NOFILL 1

/* Types of objects we support. */
#define OBJECT_NONE 0
#define OBJECT_SHAPE 1
#define OBJECT_TEXT 2
#define MAX_OBJECTS 10

#define MAX_SHAPES 4
#define FIRST_SHAPE_NUM 0

/* Supported shapes */

#define SHAPE_HEART 0
#define SHAPE_STAR  1
#define SHAPE_DIAMOND 2
#define SHAPE_FACE 3

/* Supported text messages */

#define TEXT_LOVE 0
#define TEXT_VALENTINE 1
#define TEXT_HUGS 2
#define TEXT_SWEET 3


/*Possible places for mouse clicks*/
#define CLICK_BUTTON 1  /* You clicked on SOME button (doesn't say which one)*/
#define CLICK_CARD 2    /* You clicked in the card area */
#define CLICK_COLOR 3	/* You clicked on the color menu */
#define CLICK_OUTSIDE 4 /* You clicked outside the card area, not on a button*/

/*******************  FUNCTION PROTOTYPES **************************/

/* Drawing functions */
void DrawBlankScreen (void);
void DrawButtons(int last_button);
void DrawCard (int *background_color, int *status, int *object_count, 
			   int object_type[], int shape_type[],
			   int message_type[], 
			   int object_color[], int object_x[], int object_y[]);
void DrawMessage(int message_type, int object_color, int object_x,
	int object_y);
void DrawShape(int color, int shape_type, int bottom_x, int bottom_y);
void DrawHeart (int x, int y, int color);
void DrawStar (int x, int y, int color);
void DrawDiamond (int x, int y, int color);
void DrawFace (int x, int y, int color);
void DrawColorMenu (void);

/* Mouse click handling functions */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int shape_type[], int message_type[], 
					  int object_color[], 
					  int object_x[], int object_y[], int *last_button,
					  int *selected_object);
int ClassifyMouseClick(int mouse_x, int mouse_y);
int HandleButton(int x, int y, int *status, int object_type[], int shape_type[],
	int message_type[], int *selected_object);
void HandleColorClick(int x, int y, int *status, int *background_color, int object_color[],
	int *selected_object);

void zero_array(int entry[]);	/* Zeros the arrays */
int SelectTheObject(int x, int y, int object_x[], int object_y[], int *object_count);

/* Add a shape or text to the card. */
void AddShape(int x, int y, int *status, int *object_count, int *selected_object,
			   int object_type[], int shape_type[], int object_color[], 
			   int object_x[], int object_y[]);
void AddText(int x, int y, int *status, int *object_count, int *selected_object,
	int object_type[], int message_type[], int object_color[],
	int object_x[], int object_y[]);

/*****************************************************************************

     MAIN

*****************************************************************************/

/* main() has two chief purposes: to contain the important variables and 
 *initialize them, and to contain the "event loop," which constantly looks 
 *to see if an event has happened, and then processes the event by calling 
 *an appropriate function. An example of an event is a mouse click.
 */
int main(void)
{
	int quit = FALSE;              /* becomes TRUE to exit program */
	int mouse_x, mouse_y;          /* coordinates of a mouse click */
	char key_hit;                  /* which key was hit            */
	
	
	/*variables and their default values*/
	int background_color = YELLOW;
	int status = STATUS_ADD_SHAPE;
	
	/* Each object will have a number telling what kind of object it is, a color,
	 * and an x and y coordinate.  You'll need to make these into arrays so you
	 * can support more than one object.
	 */
	int object_count = 0;           /* How many objects on the page? */
	int selected_object;		/* Which object is selected */
	int object_type[MAX_OBJECTS] ;  /* Shape or text */
	int shape_type[MAX_OBJECTS];	/* Type of shape */
	int message_type[MAX_OBJECTS];	/* Type of text */
	int object_color[MAX_OBJECTS];  /* Color of object */
	int object_x[MAX_OBJECTS];      /* Position of object */
	int object_y[MAX_OBJECTS];
	
	int last_button = 1;           /* Keep track of the last button clicked.
	                                   This would be usefull when highlighting
	                                   the last button the user clicked */

	int event; /*the most recent GP142 event*/
	time_t t; /*needed only when initializing a random number generator*/
	

	/*
	 * INITIALIZATIONS
	 */
	srand (time(&t));	    /* Seed the random number generator. This is 
							   something you should do if you plan to use 
							   random numbers. Leave it here in your program 
							   and forget about it.*/
	

	/* These GP142 initializations don't need to be changed */
	GP142_open();           /* Open and init the GP142 Graphics Window      */
	GP142_show_text(0);     /* Hide the text window                         */
	GP142_logging(LOG_OFF); /* Change to LOG_ON if you want it.             */
	
	GP142_clear();	        /* This will clear the window & paint it white. */
	GP142_animate(ANI_HALT); /* Enable periodic events                       */

	/* Zero the arrays */
			
	zero_array(object_type);
	zero_array(shape_type);
	zero_array(message_type);
	zero_array(object_color);
	zero_array(object_x);
	zero_array(object_y);
	/*
	 * The main event loop:
	 * --- ---- ----- -----
	 * wait for the next user action, such as a mouse click or keyboard
	 * keypress, or a periodic update event (allowing us to display the
	 * next frame of an animation -- not used directly in this program)
	 * and call the appropriate function to handle it.  Repeat this until 
	 * the user signals "quit".
	 */
	while (!quit) {  /*this is the famous "event loop" */
		
		/*start by redrawing everything */
		DrawBlankScreen ();
		DrawCard (&background_color, &status, &object_count, 
			      object_type, shape_type, message_type, 
			      object_color, object_x, object_y);
		DrawButtons(last_button);	  /* Show the buttons...*/ 

		DrawColorMenu();

						
		/*Wait for something to happen */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_hit);
		
		/* Something just happened.  Figure out what it was, and react 
		   appropriately. */
		switch (event) {
		case GP142_QUIT:
			quit = TRUE;           /* set flag to terminate loop           */
			break;
			
		case GP142_KBD:  /*A key was pressed */
			/*not needed for this program. Use this if user will need to 
			  hit a key. Note that scanf CANNOT be used with GP142.  If you 
			  use this, the key character value is in variable "key_hit"*/
			break;
			
		case GP142_MOUSE:
			/* A mouse click has occurred. Call the mouse handler to 
			   decide where the click was and react as needed. */
			HandleMouseClick(mouse_x, mouse_y, &background_color, &status, 
						&object_count, object_type, 
						shape_type, message_type, object_color, 
						object_x, object_y, &last_button,
						&selected_object);


			
			break;
			
		case GP142_PERIODIC:
			/*An events has occurred simply because some time has passed,*/
			/*not currently used in the program*/		
			break;

		default:
			
			break;
			
		}  /*end switch */
	}  /* end event loop */
	
	
	/*
	 * WRAP-UP
	 */
	GP142_close();         /* clean up and close graphics window   */
	
	return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/
/* 
 * For a blank screen just draw a white rectangle that covers the whole
 * GP142 window.
 */
void DrawBlankScreen (void) {
	int font_size = 14;

	GP142_rectangleXY (WHITE,
					-GP142_XMAX, -GP142_YMAX,
					GP142_XMAX, GP142_YMAX,
					FILL);
} /*end DrawBlankScreen */


/* 
 * Draws the buttons 
 */
void DrawButtons(int last_button)
{
	char b_labels [NUM_BUTTONS][25] = {
		"Background Color", "Add Shape", "Change Object",
		"Add Message", "Select Object"
			
	};
	
	int i, text_offset = -GP142_XMAX + 6, font_size = 10;
	int y, label_y;
	
	/* draw rectangles and labels for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		y = GP142_YMAX - i * BUTTON_HEIGHT;
		
		GP142_rectangleXY(BLACK,
			-GP142_XMAX + 4,
			y - 2,
			-GP142_XMAX + BUTTON_WIDTH,
			y - BUTTON_HEIGHT - 2, 1);

		label_y = (y - 2) - BUTTON_HEIGHT/2;
		
		/* If the button was pressed, highlight it */

		if (i == last_button){
			GP142_rectangleXY(MED_GRAY,
				-GP142_XMAX + 5,
				y - 3,
				-GP142_XMAX + BUTTON_WIDTH - 1,
				y - BUTTON_HEIGHT - 1, FILL);
			}

		/* If the button pressed was the Change Shape button,
		 * unhighlight it again. It shouldn't remain on. */

		if(i == 2){
 			GP142_rectangleXY(WHITE,
                                 -GP142_XMAX + 5,
		                 y - 3,
		                 -GP142_XMAX + BUTTON_WIDTH - 1,
                                 y - BUTTON_HEIGHT - 1, FILL);
			}
			GP142_printfXY(BLACK, text_offset, label_y, font_size, b_labels[i]);	
	}	
}


/*
 * Render (draw) the page and all its contents.
 */
void DrawCard (int *background_color, int *status, int *object_count, 
			   int object_type[], int shape_type[], int message_type[], 
			   int object_color[], int object_x[], int object_y[])
{
	int font_size = 14;
	int i;
	/*
	 * Draw the background of the card.
	 */
	GP142_rectangleXY (*background_color,
		PAGE_LLX, PAGE_LLY,
		PAGE_LLX+ PAGE_WIDTH, PAGE_LLY+ PAGE_HEIGHT,
		FILL);
	/* 
	 * Draw a shape on the card, if one currently exists.
	 */
	if (*object_count > 0) {
		for(i = 0; i < *object_count; i++){
			if(object_type[i] == 1){
				DrawShape(shape_type[i], object_color[i], 
					object_x[i], object_y[i]);
			}	
			if(object_type[i] == 2){
				DrawMessage(message_type[i], object_color[i],
					object_x[i], object_y[i]);
			}
		}
	}
	
	/*
	 * Write an appropriate status message below the background
	 */




	 
	switch (*status) {
	case STATUS_ADD_SHAPE:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a shape.");
		break;
	case STATUS_ADD_TEXT:
		GP142_printfXY(BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the card to add a message.");
		break;
	case STATUS_SELECTED:
		GP142_printfXY(BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Object selected. Click button to change shape, or click");
		GP142_printfXY(BLACK, PAGE_LLX, PAGE_LLY-60, font_size,		 
			"the color menu to change color.");
		break;
	case STATUS_BACKGROUND:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click on the color menu to change the background color.");
		break;
	case STATUS_SELECT:
		GP142_printfXY(BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Click near an object to select it.");
		break;
	default:
		GP142_printfXY (BLACK, PAGE_LLX, PAGE_LLY-40, font_size,
			"Status error -- notify programmer");
		break;
	} /*end switch */
	
	
} /*end DrawPage */


/*
 * Draw one shape.
 */
void DrawShape(int shape_type, int object_color, int object_x, int object_y) {

	switch (shape_type) {
	case SHAPE_HEART:
		DrawHeart(object_x, object_y, object_color);
		break;
	case SHAPE_STAR:
		DrawStar(object_x, object_y, object_color);
		break;
	case SHAPE_DIAMOND:
		DrawDiamond(object_x, object_y, object_color);
		break;
	case SHAPE_FACE:
		DrawFace(object_x, object_y, object_color);
		break;
	default:
		/* Unknown shape selected */
		break;

	} /*end switch*/

}

/* Draw a text message. */

void DrawMessage(int message_type, int object_color, int object_x, int object_y){

	switch(message_type){
		case TEXT_LOVE:
			GP142_printfXY(object_color, object_x, object_y, 14,
				"Love ya!");
			break;
		case TEXT_VALENTINE:
			GP142_printfXY(object_color, object_x, object_y, 14,
				"Be my valentine!");
			break;
		case TEXT_HUGS:
			GP142_printfXY(object_color, object_x, object_y, 14,
				"Hugs and kisses!");
			break;
		case TEXT_SWEET:
			GP142_printfXY(object_color, object_x, object_y, 14,
				"You're so sweet!");
			break;
	}


}

/* Draw a heart */

#define HEART_WIDTH 35
void DrawHeart (int x, int y, int color) {

	int h = (HEART_WIDTH/2);

	GP142_circleXY (color, x - h + 5, y, h);
	GP142_circleXY (color, x + h - 5, y, h);
	GP142_triangleXY (color, x- h*2 +5, y-5,
						x + h*2 -5, y-5,
						x, y - h*2 -5, FILL);

}

/* Draw a five pointed star */
 
#define STARLEG 30
#define PI 3.1415926
void DrawStar (int centerx, int centery, int color) {

	int x[10], y[10];
	int i;
	for(i = 0; i < 10; i++){
		if((i % 2) == 0){
			y[i] = (int)(STARLEG * cos(i * ((2.0 * PI)/10.0)) 
				+ (double)centery);
			x[i] = (int)(STARLEG * sin(i * ((2.0 * PI)/10.0)) 
				+ (double)centerx);
		}else{
			y[i] = (int)((STARLEG/2) * cos(i * ((2.0 * PI)/10.0)) 
				+ (double)centery);
			x[i] = (int)((STARLEG/2) * sin(i * ((2.0 * PI)/10.0)) 
				+ (double)centerx);
		}
	}

/* The geometry here is icky, but it works, trust me. */
	
	GP142_triangleXY (color, x[0], y[0], x[3], y[3], x[7], y[7], FILL);
	GP142_triangleXY (color, x[2], y[2], x[5], y[5], x[9], y[9], FILL);
	GP142_triangleXY (color, x[4], y[4], x[7], y[7], x[1], y[1], FILL);
	GP142_triangleXY (color, x[6], y[6], x[9], y[9], x[3], y[3], FILL);
	GP142_triangleXY (color, x[8], y[8], x[1], y[1], x[5], y[5], FILL);
}

/* Draw a diamond */

#define DIAMOND_WIDTH 40
#define DIAMOND_HEIGHT 60
void DrawDiamond (int x, int y, int color){

	GP142_triangleXY(color, x - (DIAMOND_WIDTH/2), y,
			x + (DIAMOND_WIDTH/2), y,
			x, y + (DIAMOND_HEIGHT/2), FILL);
	GP142_triangleXY(color, x - (DIAMOND_WIDTH/2), y,
			x + (DIAMOND_WIDTH/2), y,
			x, y - (DIAMOND_HEIGHT/2), FILL);
}

/* Draw a smiley face */

#define FACE_DIAMETER 60
void DrawFace (int x, int y, int color){

	/* Notice that if the user selects black as the face's color,
	 * the features automatically change to white. */

	int feature_color;
	if(color == BLACK){ 
		feature_color = WHITE;
	}else{
		feature_color = BLACK;
	}


	GP142_circleXY(color, x, y, FACE_DIAMETER/2);
	GP142_circleXY(feature_color, x - 10, y + 8, 5);
	GP142_circleXY(feature_color, x + 10, y + 8, 5);
	GP142_lineXY(feature_color, x - 5, y - 15, x + 5, y - 15, 1);
	GP142_lineXY(feature_color, x - 5, y - 15, x - 10, y - 11, 1);
	GP142_lineXY(feature_color, x + 5, y - 15, x + 10, y - 11, 1);
	GP142_lineXY(feature_color, x - 10, y - 11, x - 15, y - 4, 1);
	GP142_lineXY(feature_color, x + 10, y - 11, x + 15, y - 4, 1);
}

/*
 * Mouse clicks occur in one of four regions of interest: on buttons, 
 * on the card, on the color menu, or anywhere else.
 */
int  ClassifyMouseClick(int mouse_x, int mouse_y)
{
  if ((mouse_x < -GP142_XMAX + BUTTON_WIDTH) &&
      (mouse_x > -GP142_XMAX + 3) &&
      (mouse_y < GP142_YMAX) &&
      (mouse_y > (GP142_YMAX - NUM_BUTTONS*BUTTON_HEIGHT)))
    return CLICK_BUTTON;
  else if (mouse_x >= PAGE_LLX && mouse_x < PAGE_LLX + PAGE_WIDTH
	  && mouse_y >= PAGE_LLY && mouse_y < PAGE_LLY + PAGE_HEIGHT)
    return CLICK_CARD;
  else if (mouse_y <= 0 && mouse_y >= (-COLOR_SIDE * 6) &&
	   (mouse_x > -GP142_XMAX + 3) && 
	   (mouse_x < -GP142_XMAX + 3 + (COLOR_SIDE * 4)))
    return CLICK_COLOR;
  else
    return CLICK_OUTSIDE;

}


/*
 * Decide where a mouse click occured, and react appropriately.
 */
void HandleMouseClick(int mouse_x, int mouse_y, int *background_color, 
					  int *status, int *object_count, int object_type[], 
					  int shape_type[], int message_type[], int object_color[], 
					  int object_x[], int object_y[], int *last_button,
					  int *selected_object)
{

	int mouse_click;
	int test;



	
	mouse_click = ClassifyMouseClick(mouse_x, mouse_y);
			
	if (mouse_click == CLICK_BUTTON) {
				
		/* A button has been clicked.  Call a function to handle it*/
		/* most of the variables passed as args will NOT change*/

		*last_button = HandleButton(mouse_x, mouse_y, status, 
					 object_type, shape_type, message_type,
					 selected_object);

	} else if (mouse_click == CLICK_CARD) {
		switch(*status){
			case STATUS_ADD_TEXT:
				AddText(mouse_x, mouse_y, status, object_count,
					selected_object, object_type, message_type, 
					object_color, object_x, object_y);
				break;
			case STATUS_ADD_SHAPE:
				AddShape(mouse_x, mouse_y, status, object_count,
					selected_object, object_type, shape_type,
					object_color, object_x, object_y);
				break;
			case STATUS_SELECT:
			/* Check to see that there is a selected object.
			 * Then make it the selected object for real. */
				test = SelectTheObject(mouse_x, mouse_y, object_x,
					object_y, object_count);
				if(test <= 10 && test >= 0){
					*selected_object = test;
					*status = STATUS_SELECTED;
				}
				break;
			case STATUS_SELECTED:
				object_x[*selected_object] = mouse_x;
				object_y[*selected_object] = mouse_y;
				break;
		}



	} else if(mouse_click == CLICK_COLOR){
		HandleColorClick(mouse_x, mouse_y, status, background_color, object_color,
		selected_object);
	}	
	/* Ignore any other mouse clicks */
}

/* 
 * Figure out which button was clicked, if any, and do the right 
 * thing.
 */
int HandleButton(int x, int y, int *status, int object_type[], int shape_type[],
			int message_type[], int *selected_object)
{
	int buttonNum = 0;
	
	/* If we're outside of this area, we could not have clicked on
	 * a button.
	 */

	if (x > -GP142_XMAX + BUTTON_WIDTH){  
		return (999);	/*That's an error situation*/
	}
	
	/* We know the mouse clicked in the column of buttons. Calculate
	 * which one.
	 */

	buttonNum = (GP142_YMAX - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* Select the background to change its color */
		*status = STATUS_BACKGROUND;
		break;
	case 1:
		/* Set status to be able to add a shape */
		*status = STATUS_ADD_SHAPE;
		break;
	case 2:
		/* Change selected shape if it is selected */
		if(*status == STATUS_SELECTED && 
			object_type[*selected_object] == OBJECT_SHAPE){
			if(shape_type[*selected_object] == 3){
				shape_type[*selected_object] = 0;
			}else{
				shape_type[*selected_object] = 
					shape_type[*selected_object] + 1;
			}
		}
		if(*status == STATUS_SELECTED &&
			object_type[*selected_object] == OBJECT_TEXT){
			if(message_type[*selected_object] == 3){
				message_type[*selected_object] = 0;
			}else{
				message_type[*selected_object] = 
					message_type[*selected_object] + 1;
			}
		}
		break;
	case 3:
		/* Set status to be able to add a message */
		*status = STATUS_ADD_TEXT;
		break;
	case 4:
		/* Set status to select objects */
		*status = STATUS_SELECT;
		break;
	default:
		*status = 999;
		/*Bad news if this is ever reached! */
		break;
	}
	return buttonNum;
}


/* If there's room for another shape, add it to the card. */
void AddShape(int x, int y, int *status, int *object_count, int *selected_object, 
			   int object_type[], int shape_type[], int object_color[], 
			   int object_x[], int object_y[])
{
	if (*object_count <= MAX_OBJECTS) {
		*status = STATUS_SELECTED;
		object_type[*object_count] = OBJECT_SHAPE;
		shape_type[*object_count] = SHAPE_HEART;
		object_color[*object_count] = RED;
		object_x[*object_count] = x;
		object_y[*object_count] = y;
		*selected_object = *object_count;
		*object_count = *object_count + 1;
	}
}

/* If there's room for another object, add a text message to the card. */

void AddText(int x, int y, int *status, int *object_count, int *selected_object,
		int object_type[], int message_type[], int object_color[],
		int object_x[], int object_y[]){

	if(*object_count <= MAX_OBJECTS){
		*status = STATUS_SELECTED;
		object_type[*object_count] = OBJECT_TEXT;
		message_type[*object_count] = TEXT_VALENTINE;
		object_color[*object_count] = RED;
		object_x[*object_count] = x;
		object_y[*object_count] = y;
		*selected_object = *object_count;
		*object_count = *object_count + 1;
	}
}

/* This draws that color menu in a 4x6 block. Each block is one color
 * more than the last one. */

void DrawColorMenu(void){

	int side = COLOR_SIDE;
	int x = -GP142_XMAX + 4;
	int y = 0;
	int color = 0;
	int i, j;

	for(i = 0; i < 6; i++){
		for(j = 0; j < 4; j++){
			x = x + j * side;
			GP142_rectangleXY(color, x, y,
				x + side + 1, y - side, FILL);

				GP142_rectangleXY(BLACK, x, y,
					x + side + 1, y - side, 1);
			x = -GP142_XMAX + 4;
			color = color + 1;
			}
		y = (y - side);
		}
}

/* This function handles any clicks that might be in the color menu. 
 * Depending on the status, and which square the click was in, the appropriate
 * color is changed accordingly. */

void HandleColorClick(int x, int y, int *status, int *background_color, int object_color[],
			int *selected_object){

	int side = COLOR_SIDE;
	int color = 0;
	int color_select, i, j;
	for(i = 0; i < 6; i++){
		for(j = 0; j < 4; j++){
			if((x > (-GP142_XMAX + 3 + (j * side))) && 
				(x < (-GP142_XMAX + 3 + ((j + 1) * side))) &&
				(y < (i * -side)) && (y > ((i + 1) * -side))){
					color_select = color;
			}
			color = color + 1;
		}
	}
	switch(*status){
		case STATUS_BACKGROUND:
			*background_color = color_select;
			break;
		case STATUS_SELECTED:
			object_color[*selected_object] = color_select;
			break;
		default:
			break;
	}
}

/* This function selects an object on the card. It takes the mouse coordinates
 * and compares them with the coordinate arrays of the objects. If one of them
 * is within a certain amount, it returns that index value. If more than one
 * are within that range, it narrows the range. */

int SelectTheObject(int x, int y, int object_x[], int object_y[], int *object_count){

	int count = 0;
	int selected_object, i;
	
	/* Do it once */
	
	for(i = 0; i < *object_count; i++){
		if((abs(x - object_x[i])) < 20 && (abs(y - object_y[i])) < 20){
			count = count + 1;
			selected_object = i;
		}
	}

	/* If that didn't work, do it again */
	
	if(count > 1){
		count = 0;
		for(i = 0; i < *object_count; i++){
			if((abs(x - object_x[i])) < 10 && (abs(y - object_y[i])) < 10){
				count = count + 1;
				selected_object = i;
			}
		}
	}

	/* And if THAT didn't work, try it one more time */
	
	if(count > 1){
		count = 0;
		for(i = 0; i < *object_count; i++){
			if((abs(x - object_x[i])) < 5 && (abs(y - object_y[i])) < 5){
				count = count + 1;
				selected_object = i;
			}
		}
	}
	if(count == 0){ 
		return(999);	/* Returns 999 if nothing is close to the click */
	}else{
		return selected_object;
	}
}

/* Clear out the arrays */

void zero_array(int entry[]){
	int i;
	
	for(i = 0; i < MAX_OBJECTS; i++){
		entry[i] = 0;
	}
}

